//   

#include<stdio.h>
#include<memory>

#define NODE_SIZE (4)	//!<    

class ArrayTree{
protected:
	struct node_st{
		int array[NODE_SIZE]; 
		node_st *p1,*p2;	
		int count;	//         UNDEF ,      array   
	};
	node_st *root;						//!<  
	void Dump(FILE*,node_st*,int);		//!<    
	void Clear(node_st*);				//!<  
	void FixNode(node_st**);			//!<    
	node_st *NewNode(int);				//!<   
	node_st *GetMin(node_st*,int&);		//!<  
	node_st *GetMax(node_st*,int&);		//!<       
	bool InsertToNode(node_st*,int&);	//!<  ( )   
	bool Insert(node_st**,int);			//!<    
	bool Remove(node_st**,int);			//!<     
public:
	ArrayTree();
	~ArrayTree();
	bool Insert(int value);		//!<  
	bool Remove(int value);		//!< 
	void Dump(FILE * =stdout);	//!<  
};

ArrayTree::ArrayTree()
{
	root=0;
}

ArrayTree::~ArrayTree()
{
	Clear(root);
}

//!     
void ArrayTree::Clear(node_st *node)
{
	if(!node) return;
	Clear(node->p1);
	Clear(node->p2);
	delete node;
}

//!        
//! \param value    
ArrayTree::node_st *ArrayTree::NewNode(int value)
{
	node_st *node=new node_st;
	node->count=1;
	node->p1=node->p2=0;
	node->array[0]=value;	
	return node;
}

//! \retval true -    
bool ArrayTree::Insert(int value)
{
	return Insert(&root,value);
}

//! \retval true -    
bool ArrayTree::Remove(int value)
{
	return Remove(&root,value);
}

//! \param value 
//! \param    
//! \retval    ,       
//! \retval true -    
bool ArrayTree::Insert(node_st **p,int value)
{
	node_st *node=*p;
	if(!node) {*p=NewNode(value);return false;}
	int n=node->count;
	if(InsertToNode(node,value)) return true;
	if(n<NODE_SIZE) return false;
	if(value<node->array[0]) return Insert(&node->p1,value); 
	else return Insert(&node->p2,value);
}

//! \param value 
//! \param    
//! \retval true      
bool ArrayTree::InsertToNode(node_st *node,int &value)
{
	int n,m,res;
	if(node->count==NODE_SIZE) {
		if(value<node->array[0]) return false;
		if(value>node->array[NODE_SIZE-1]) return false;
	}
	//   
	for(n=0; n<node->count; n++) {
		if(value==node->array[n]) return true;
		if(value<node->array[n]) break;
	}
	if(node->count<NODE_SIZE /*|| rand()&1*/) {	//     
		res=node->array[NODE_SIZE-1];
		for(m=NODE_SIZE-1; m>n; m--) node->array[m]=node->array[m-1];
		if(n<NODE_SIZE) node->array[n]=value,value=res; 
	} else {
		res=node->array[0]; n--;
		for(m=0; m<n; m++) node->array[m]=node->array[m+1];
		if(n<0) value=res; else node->array[n]=value,value=res;
	}
	if(node->count<NODE_SIZE) node->count++;
	return false;
}

//! \param value 
//! \param    
//! \retval    ,      
//! \retval true -    
bool ArrayTree::Remove(node_st **p,int value)
{
	int n;
	node_st *node=*p;
	if(!node) return true;
	if(node->count>=NODE_SIZE) {
		if(value<node->array[0]) return Remove(&node->p1,value);
		if(value>node->array[node->count-1]) return Remove(&node->p2,value);
	}
	for(n=0; n<node->count; n++) if(node->array[n]==value) break;
	if(n==node->count) return true;
	for(;n<node->count-1; n++) node->array[n]=node->array[n+1];
	node->count--;
	FixNode(p);
	return false;
}

//! \param node   
//! \result    ,      
void ArrayTree::FixNode(node_st **p)
{
	node_st *node=*p;
	if(!node->count) {delete node;*p=0;return;}
	if(node->count==NODE_SIZE) return;
	if(node->p1 /*|| (node->p1 && node->p2 && rand()&1)*/) {	//     
		for(int n=NODE_SIZE-1; n; n--) node->array[n]=node->array[n-1];
		node->p1=GetMax(node->p1,node->array[0]);
		node->count++;
	}else if(node->p2) {
		node->p2=GetMin(node->p2,node->array[NODE_SIZE-1]);
		node->count++;
	}
}

//! \param node   
//! \retval  
//! \result    ,      
ArrayTree::node_st *ArrayTree::GetMin(node_st *node,int &min)
{
	if(node->p1) {node->p1=GetMin(node->p1,min);return node;}
	min=node->array[0]; 
	for(int n=1; n<node->count; n++) node->array[n-1]=node->array[n];
	node->count--;
	FixNode(&node);
	return node;
}

//! \param node   
//! \retval  
//! \result    ,      
ArrayTree::node_st *ArrayTree::GetMax(node_st *node,int &max) 
{
	while(node->p2) {node->p2=GetMax(node->p2,max);return node;}
	node->count--;
	max=node->array[node->count];
	FixNode(&node);
	return node;
}

void ArrayTree::Dump(FILE *file)
{
	fprintf(file,"\n[dump]\n");
	Dump(file,root,0);
}

void ArrayTree::Dump(FILE *file,node_st *node,int depth)
{
	if(!node) return;
	Dump(file,node->p1,depth+1);
	printf("%*s[",depth*2,"");
	for(int n=0; n<node->count; n++) {
		if(n) fputc(' ',file);
		fprintf(file,"%d",node->array[n]);
	}
	fprintf(file,"]\n");
	Dump(file,node->p2,depth+1);
}


void main()
{
	int n,v;
	bool array[100]; memset(array,0,100);
	ArrayTree tree;
	for(n=1; n<=500; n++) {
		v=rand()%100; printf(" +[%d]",v); tree.Insert(v); array[v]=true;
		v=rand()%100; printf(" -[%d]",v); tree.Remove(v); array[v]=false;
	}
	tree.Dump();
	getchar();
}
